import os

from dotenv import load_dotenv

load_dotenv()


def public_folder(path=""):
    ensure_public_folder()
    base_dir = "public"
    return apply_root_folder(path=base_dir, subpath=path)


def catalogs_folder(path=""):
    ensure_catalogs_folder()
    base_dir = "catalogs"
    return apply_root_folder(path=base_dir, subpath=path)


def templates_folder(path=""):
    ensure_template_folder()
    base_dir = "templates"
    return apply_root_folder(path=base_dir, subpath=path)


def apply_root_folder(path, subpath=""):
    root = os.getenv("BKMRKS_DIR_ROOT", ".")
    dir_name = os.path.join(root, path, subpath)
    return dir_name


def ensure_public_folder():
    dir_to_ensure = apply_root_folder(path="public")
    if not os.path.exists(dir_to_ensure):
        os.mkdir(dir_to_ensure)


def ensure_template_folder():
    dir_to_ensure = apply_root_folder(path="templates")
    if not os.path.exists(dir_to_ensure):
        os.mkdir(dir_to_ensure)
        default_templates_dir = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "templates"
        )
        files = os.listdir(default_templates_dir)
        for file in files:
            if file.split(".")[1] in ["css", "html"]:
                with open(os.path.join(default_templates_dir, file), "r+") as fr:
                    with open(os.path.join(dir_to_ensure, file), "+w") as fw:
                        fw.write(fr.read())


def ensure_catalogs_folder():
    dir_to_ensure = apply_root_folder(path="catalogs")
    if not os.path.exists(dir_to_ensure):
        from bkmrks import bkmrks

        os.mkdir(dir_to_ensure)
        data = {
            bkmrks.create_line_name(line_index=1): {
                bkmrks.create_item_name(item_index=1): bkmrks.get_bookmark_item(
                    url="https://github.com/bouli/bkmrks",
                    name="bkmrks_sample_page",
                    img="https://cesarcardoso.cc/README/1_bouli.png",
                )
            }
        }
        bkmrks.set_catalog_data(data=data)



import os


def apply_ext(file_path: str, ext: str) -> str:
    file_name = extract_file_name_no_ext(file_path=file_path)
    if file_name != "":
        file_name = ".".join([file_name] + [ext])
    file_path = os.path.join(os.path.dirname(file_path), file_name)
    return file_path


def extract_file_name_no_ext(file_path: str) -> str:
    file_name = os.path.basename(file_path)

    file_name_elements = file_name.split(".")
    if len(file_name_elements) > 1:
        file_name_elements = file_name_elements[:-1]

    file_name = ".".join(file_name_elements)
    return file_name


def extract_ext(file_path: str) -> str:
    file_name = os.path.basename(file_path)

    file_name_elements = file_name.split(".")
    if len(file_name_elements) > 1:
        ext = file_name_elements[-1]
    else:
        ext = ""
    return ext



from bkmrks import files

def test_apply_ext():
    tests = []
    tests.append({"ext":"yaml", "file_path":"", "expected_result":""})
    tests.append({"ext":"yaml", "file_path":"/", "expected_result":"/"})
    tests.append({"ext":"yaml", "file_path":"/folder/", "expected_result":"/folder/"})
    tests.append({"ext":"yaml", "file_path":"/folder/subfolder/", "expected_result":"/folder/subfolder/"})

    tests.append({"ext":"yaml", "file_path":"/folder/fi.le.py", "expected_result":"/folder/fi.le.yaml"})
    tests.append({"ext":"yaml", "file_path":"/folder/file.py", "expected_result":"/folder/file.yaml"})
    tests.append({"ext":"yaml", "file_path":"/folder/file", "expected_result":"/folder/file.yaml"})
    tests.append({"ext":"yaml", "file_path":"/file.py", "expected_result":"/file.yaml"})
    tests.append({"ext":"yaml", "file_path":"file.py", "expected_result":"file.yaml"})
    tests.append({"ext":"yaml", "file_path":"file", "expected_result":"file.yaml"})

    for test in tests:
        assert files.apply_ext(ext=test["ext"], file_path=test["file_path"]) == test["expected_result"]


def test_extract_file_name_no_ext():
    tests = []
    tests.append({"file_path":"", "expected_result":""})
    tests.append({"file_path":"/", "expected_result":""})
    tests.append({"file_path":"/folder/", "expected_result":""})
    tests.append({"file_path":"/folder/subfolder/", "expected_result":""})

    tests.append({"file_path":"/folder/fi.le.py", "expected_result":"fi.le"})
    tests.append({"file_path":"/folder/file.py", "expected_result":"file"})
    tests.append({"file_path":"/folder/file", "expected_result":"file"})
    tests.append({"file_path":"/file.py", "expected_result":"file"})
    tests.append({"file_path":"file.py", "expected_result":"file"})
    tests.append({"file_path":"file", "expected_result":"file"})

    for test in tests:
        assert files.extract_file_name_no_ext(file_path=test["file_path"]) == test["expected_result"]

def test_extract_ext():
    tests = []
    tests.append({"file_path":"", "expected_result":""})
    tests.append({"file_path":"/", "expected_result":""})
    tests.append({"file_path":"/folder/", "expected_result":""})
    tests.append({"file_path":"/folder/subfolder/", "expected_result":""})
    tests.append({"file_path":"/folder/fi.le.py", "expected_result":"py"})
    tests.append({"file_path":"/folder/file.py", "expected_result":"py"})
    tests.append({"file_path":"/folder/file", "expected_result":""})
    tests.append({"file_path":"/file.py", "expected_result":"py"})
    tests.append({"file_path":"file.py", "expected_result":"py"})
    tests.append({"file_path":"file", "expected_result":""})

    for test in tests:
        assert files.extract_ext(file_path=test["file_path"]) == test["expected_result"]



from urlicon import urlicon

from bkmrks import bkmrks, files, folders, urls


def generate(md_file_name=None, catalog="index"):
    if md_file_name is None:
        md_file_name = folders.public_folder(path="index")

    bookmarks = bkmrks.get_catalog_data(catalog=catalog)
    md_file_content = ""
    if len(bookmarks) > 0:
        # for line in bookmarks.values():
        for line_name, line in bookmarks.items():
            if len(line) > 0:
                line_alias = bkmrks.get_alias_from_line_name(line_name)
                md_file_content += md_h3(line_alias)
                for item in line.values():
                    md_file_content += md_a_img(item)
                md_file_content += md_hr()
    if len(md_file_content) < 10:
        return None

    md_file_name = files.apply_ext(md_file_name, ext="md")

    with open(md_file_name, "+w") as f:
        f.write(md_file_content)
    return md_file_name


def md_a_img(item):
    try:
        url = item["url"]
    except:
        return ""
    if "name" in item:
        name = item["name"]
    else:
        name = urls.get_name_from_domain(url=url)

    if "img" in item:
        img = item["img"]
    else:
        img = urlicon.get_default_img(text=name)

    return f'\n[![{name}]({img})]({url} "{name}")'


def md_hr():
    return "\n---"


def md_h3(text):
    if len(text) == 0:
        return ""
    return f"\n### {text}"



import argparse

from bkmrks import bkmrks, presenter


def main():
    parser = argparse.ArgumentParser(prog="bkmrks")
    parser.add_argument("--version", action="version", version="%(prog)s v0.5.4")

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    ## render
    render_parser = subparsers.add_parser(
        "render",
        help="✍️   Render your html bookmarks to public folder.",
    )

    ## load
    load_parser = subparsers.add_parser(
        "load",
        help="📚 Load and HTML file/url and create a bookmark page.",
    )

    load_parser.add_argument(
        "url_or_html_file",
        help="HTML file/url to scrape.",
    )

    load_parser.add_argument(
        "-catalog",
        "-c",
        help="Bookmark catalog name.",
        default="index",
    )

    ## add
    add_parser = subparsers.add_parser(
        "add",
        help="✍️   Add a bookmark to catalog.",
    )

    add_parser.add_argument(
        "url",
        help="URL bookmark to add to the catalog.",
    )

    add_parser.add_argument(
        "-catalog",
        "-c",
        help="Bookmark catalog name.",
        default="index",
    )

    add_parser.add_argument(
        "-line",
        "-l",
        help="Line of the catalog to add your bookmark.",
        default="1",
    )

    add_parser.add_argument(
        "-bookmark_item",
        "-b",
        help="Bookmark item in the line of the catalog to add your bookmark.",
        default="0",
    )

    ## rm
    rm_parser = subparsers.add_parser(
        "rm",
        help="❌ Remove a bookmark from catalog.",
    )

    rm_parser.add_argument(
        "-catalog",
        "-c",
        help="Bookmark catalog name.",
        default="index",
    )

    rm_parser.add_argument(
        "-line",
        "-l",
        help="Line of the catalog to remove your bookmark.",
        default="1",
    )

    rm_parser.add_argument(
        "-bookmark_item",
        "-b",
        help="Bookmark item in the line of the catalog to remove your bookmark.",
        default="1",
    )

    ## mv
    mv_parser = subparsers.add_parser(
        "mv",
        help="🔄 Move a bookmark from a catalog to another.",
    )

    mv_parser.add_argument(
        "-default_catalog",
        "-c",
        help="Default Bookmark catalog name to move from and to.",
        default="index",
    )

    mv_parser.add_argument(
        "-default_line",
        "-l",
        help="Default line of the catalogs.",
        default="1",
    )

    mv_parser.add_argument(
        "-default_bookmark_item",
        "-b",
        help="Default Bookmark item in the line of the catalogs.",
        default=1,
    )

    mv_parser.add_argument(
        "-from_catalog",
        "-fc",
        help="Bookmark catalog name to move from.",
        default=None,
    )

    mv_parser.add_argument(
        "-from_line",
        "-fl",
        help="Line of the catalog to move from.",
        default=None,
    )

    mv_parser.add_argument(
        "-from_bookmark_item",
        "-fb",
        help="Bookmark item in the line of the catalog to move from.",
        default=None,
    )

    mv_parser.add_argument(
        "-to_catalog",
        "-tc",
        help="Bookmark catalog name to move to.",
        default=None,
    )

    mv_parser.add_argument(
        "-to_line",
        "-tl",
        help="Line of the catalog to move to.",
        default=None,
    )

    mv_parser.add_argument(
        "-to_bookmark_item",
        "-tb",
        help="Bookmark item in the line of the catalog to move to.",
        default=None,
    )

    ## mvl
    mvl_parser = subparsers.add_parser(
        "mvl",
        help="🔄 Move a line from a catalog to another.",
    )

    mvl_parser.add_argument(
        "-default_catalog",
        "-c",
        help="Default Bookmark catalog name to move from and to.",
        default="index",
    )

    mvl_parser.add_argument(
        "-from_catalog",
        "-fc",
        help="Line catalog name to move from.",
        default=None,
    )

    mvl_parser.add_argument(
        "-line",
        "-l",
        "-fl",
        help="Line of the catalog to move from.",
        default=None,
    )

    mvl_parser.add_argument(
        "-to_catalog",
        "-tc",
        help="Catalog name to move to. (default is the same of `-from_catalog`)",
        default=None,
    )

    mvl_parser.add_argument(
        "-new_alias",
        "-a",
        "-na",
        "-tl",
        help="New alias to assume in the catalog.",
        default=None,
    )

    args = parser.parse_args()
    if args.command == "render":
        presenter.render()

    if args.command == "load":
        bkmrks.html2catalog(html_file_name=args.url_or_html_file, catalog=args.catalog)
        presenter.render()

    if args.command == "add":
        bkmrks.add_bookmark(
            url=str(args.url),
            catalog=str(args.catalog),
            line_index=args.line,
            item_index=args.bookmark_item,
        )
        presenter.render()

    if args.command == "rm":
        bkmrks.remove_bookmark(
            catalog=args.catalog,
            line_index=args.line,
            item_index=args.bookmark_item,
        )
        presenter.render()

    if args.command == "mv":
        from_catalog = args.default_catalog
        to_catalog = args.default_catalog

        from_line = args.default_line
        to_line = args.default_line

        from_bookmark_item = args.default_bookmark_item
        to_bookmark_item = args.default_bookmark_item

        if args.from_catalog is not None:
            from_catalog = args.from_catalog
        if args.to_catalog is not None:
            to_catalog = args.to_catalog

        if args.from_line is not None:
            from_line = args.from_line
        if args.to_line is not None:
            to_line = args.to_line

        if args.from_bookmark_item is not None:
            from_bookmark_item = args.from_bookmark_item
        if args.to_bookmark_item is not None:
            to_bookmark_item = args.to_bookmark_item

        bkmrks.move_bookmark(
            from_catalog=from_catalog,
            from_line_index=from_line,
            from_item_index=from_bookmark_item,
            to_catalog=to_catalog,
            to_line_index=to_line,
            to_item_index=to_bookmark_item,
        )
        presenter.render()

    if args.command == "mvl":
        from_catalog = args.default_catalog
        to_catalog = args.default_catalog

        if args.from_catalog is not None:
            from_catalog = args.from_catalog
        if args.to_catalog is not None:
            to_catalog = args.to_catalog

        bkmrks.move_line(
            from_catalog=from_catalog,
            from_line_index=args.line,
            to_catalog=to_catalog,
            new_line_alias=args.new_alias,
        )
        presenter.render()

    return


if __name__ == "__main__":
    main()



import os

import yaml
from bs4 import BeautifulSoup
from urlicon import urlicon

from bkmrks import files, folders, urls


def get_catalog_data(catalog="index"):
    catalog = files.apply_ext(catalog, ext="yaml")
    catalog_path = os.path.join(folders.catalogs_folder(), catalog)

    if not os.path.exists(catalog_path):
        return {}

    with open(catalog_path, "r") as f:
        catalog_data = yaml.safe_load(f.read())
        if catalog_data is None:
            return {}
        else:
            return catalog_data


def set_catalog_data(data, catalog="index"):
    catalog = files.apply_ext(catalog, ext="yaml")
    catalog_path = os.path.join(folders.catalogs_folder(), catalog)

    with open(catalog_path, "+w") as f:
        yaml.dump(data, f)
    return get_catalog_data(catalog=catalog)


def html2catalog(html_file_name, catalog):
    domain = urls.extract_domain_from_url(url=html_file_name)
    html = urls.read_from_url_or_path(url_path=html_file_name)

    soup = BeautifulSoup(html, features="html.parser")
    soup_all_hr_a_tags = soup.find_all(["hr", "a"])

    line_index = 1
    item_index = 1
    catalog_data = {}
    line_name = create_line_name(line_index=line_index)
    catalog_data[line_name] = {}

    for soup_item in soup_all_hr_a_tags:
        if soup_item.has_attr("href") and not soup_item["href"].startswith("#"):

            img = urlicon.get_img_from_a_soup_item(soup_item=soup_item, domain=domain)
            url = urls.ensure_domain(url=soup_item["href"], domain=domain)
            name = urls.get_name_from_domain(url=url)

            bookmark_item = get_bookmark_item(url=url, name=name, img=img)

            item_name = create_item_name(item_index=item_index)
            catalog_data[line_name][item_name] = bookmark_item.copy()

            item_index += 1

    set_catalog_data(data=catalog_data, catalog=catalog)


def move_bookmark(
    from_catalog="index",
    from_line_index=1,
    from_item_index=0,
    to_catalog=None,
    to_line_index=1,
    to_item_index=0,
):

    url = get_url(
        catalog=from_catalog,
        line_index=from_line_index,
        item_index=from_item_index,
    )

    if url is None:
        return

    if to_catalog is None:
        to_catalog = from_catalog

    to_item_alias = None

    if type(to_item_index) == int and type(from_item_index) == str:
        to_item_alias = from_item_index

    remove_bookmark(
        catalog=from_catalog,
        line_index=from_line_index,
        item_index=from_item_index,
    )
    add_bookmark(
        url=url,
        catalog=to_catalog,
        line_index=to_line_index,
        item_index=to_item_index,
        item_alias=to_item_alias,
    )

    refresh_catalog_indexes(from_catalog)
    if from_catalog != to_catalog:
        refresh_catalog_indexes(to_catalog)
    return True


def add_bookmark(url, catalog="index", line_index=1, item_index=1, item_alias=None):
    catalog_data = get_catalog_data(catalog=catalog)
    line_index, line_alias = get_line_index_alias_from_catalog(
        line_index_alias=line_index, catalog_data=catalog_data
    )
    try:
        item_index = int(item_index)
        item_index = at_least_1(item_index)
        item_alias = item_alias
    except:
        item_alias = item_index
        item_index = 1
    new_item = create_item_from_url(url=url, alias=item_alias)
    new_catalog_data = {}

    if len(catalog_data) < line_index:
        line_index = len(catalog_data) + 1
        new_line_name = create_line_name(line_index=line_index, line_alias=line_alias)
        new_item_name = create_item_name(item_index=1)

        new_catalog_data = catalog_data.copy()
        new_catalog_data[new_line_name] = {}
        new_catalog_data[new_line_name][new_item_name] = new_item
    else:
        line_to_add_name = get_dict_key_by_index(
            dict_index=line_index, dict_data=catalog_data
        )
        for line_name in catalog_data.keys():
            line_items_list = list(catalog_data[line_name].values())
            if line_name == line_to_add_name:
                line_items_list.insert(item_index - 1, new_item)

            new_catalog_data[line_name] = list2line_items(
                line_items_list=line_items_list
            )
    set_catalog_data(data=new_catalog_data, catalog=catalog)
    return True


def remove_bookmark(catalog="index", line_index=1, item_index=0):
    catalog_data = get_catalog_data(catalog=catalog)
    line_index, line_alias = get_line_index_alias_from_catalog(
        line_index_alias=line_index, catalog_data=catalog_data
    )
    item_index, item_alias = get_item_index_alias_from_catalog(
        item_index_alias=item_index,
        line_index_alias=line_index,
        catalog_data=catalog_data,
    )

    line_name = get_dict_key_by_index(dict_index=line_index, dict_data=catalog_data)
    if line_name is None:
        return False

    item_name = get_dict_key_by_index(
        dict_index=item_index, dict_data=catalog_data[line_name]
    )
    if item_name is None:
        return False

    catalog_data[line_name].pop(item_name)
    set_catalog_data(data=catalog_data, catalog=catalog)

    return True


def move_line(
    from_catalog: str = "index",
    from_line_index=1,
    to_catalog=None,
    new_line_alias=None,
):
    if to_catalog == None:
        to_catalog = from_catalog
    from_catalog_data = get_catalog_data(catalog=from_catalog)
    from_line_index, from_line_alias = get_line_index_alias_from_catalog(
        line_index_alias=from_line_index, catalog_data=from_catalog_data
    )
    from_line_name = get_dict_key_by_index(
        dict_index=from_line_index, dict_data=from_catalog_data
    )

    if new_line_alias is None:
        new_line_alias = from_line_alias

    if from_line_name not in from_catalog_data:
        return
    new_line = from_catalog_data[from_line_name]

    from_catalog_data.pop(from_line_name)
    set_catalog_data(data=from_catalog_data, catalog=from_catalog)
    add_line_to_catalog(
        catalog=to_catalog, new_line=new_line, new_line_alias=new_line_alias
    )

    refresh_catalog_indexes(from_catalog)
    if from_catalog != to_catalog:
        refresh_catalog_indexes(to_catalog)
    return True


def add_line_to_catalog(catalog, new_line, new_line_alias=""):
    new_line_name = create_line_name(line_index=0, line_alias=new_line_alias)

    catalog_data = dict({new_line_name: new_line}, **get_catalog_data(catalog=catalog))
    set_catalog_data(data=catalog_data, catalog=catalog)


def refresh_catalog_indexes(catalog):
    catalog_data = get_catalog_data(catalog=catalog)
    new_catalog_data = {}
    for new_line_index, line_name in enumerate(catalog_data, start=1):
        if str(line_name).find("_") >= 0:
            new_line_alias = line_name.split("_")[1]
        else:
            new_line_alias = ""
        new_line_name = create_line_name(
            line_index=new_line_index, line_alias=new_line_alias
        )
        new_catalog_data[new_line_name] = catalog_data[line_name].copy()
    set_catalog_data(data=new_catalog_data, catalog=catalog)


def get_url(
    catalog="index",
    line_index=1,
    item_index=1,
):
    url = None
    catalog_data = get_catalog_data(catalog=catalog)
    line_index, line_alias = get_line_index_alias_from_catalog(
        line_index_alias=line_index, catalog_data=catalog_data
    )

    item_index, item_alias = get_item_index_alias_from_catalog(
        item_index_alias=item_index,
        line_index_alias=line_index,
        catalog_data=catalog_data,
    )

    if len(catalog_data) == 0:
        return
    if len(list(catalog_data.values())) >= line_index:
        catalog_line = list(catalog_data.values())[line_index - 1]
        if len(list(catalog_line.values())) >= item_index:
            if "url" in list(catalog_line.values())[item_index - 1]:
                url = list(catalog_line.values())[item_index - 1]["url"]

    return url


def create_line_name(line_index, line_alias=""):
    line_name = f"line{line_index:04d}"
    if len(line_alias) > 0:
        line_name += f"_{line_alias}"
    return line_name


def create_item_name(item_index):
    item_name = f"item{item_index:04d}"
    return item_name


def create_item_from_url(url, domain=None, alias=None):
    if domain is not None:
        url = urls.ensure_domain(url=url, domain=domain)
    if alias is not None and type(alias) != int:
        name = alias
    else:
        name = urls.get_name_from_domain(url=url)
    img = urlicon.get_url_icon(url=url)

    bookmark_item = get_bookmark_item(url=url, name=name, img=img)
    return bookmark_item


def get_bookmark_item(url, name, img):
    bookmark_item = {}
    bookmark_item["name"] = name
    bookmark_item["url"] = url
    bookmark_item["img"] = img

    return bookmark_item


def get_line_index_alias_from_catalog(line_index_alias, catalog_data):
    catalog_lines = list(catalog_data.keys())

    try:
        line_index = int(line_index_alias)
        line_index = at_least_1(line_index)
        if len(catalog_lines) >= line_index:
            line_alias = catalog_lines[line_index - 1][9:]
        else:
            line_alias = ""
    except:
        line_alias = str(line_index_alias)
        line_index = len(catalog_lines) + 1

        for catalog_line_index, catalog_line_name in enumerate(catalog_lines, start=1):
            if line_alias == catalog_line_name[9:]:
                line_index = catalog_line_index
                break
    return line_index, line_alias


def list2line_items(line_items_list: list) -> dict:
    items = {}
    for item_index, item in enumerate(line_items_list, start=1):
        item_name = create_item_name(item_index)
        items[item_name] = item
    return items


def at_least_1(number):
    number = int(number)
    if number < 1:
        number = 1

    return number


def get_dict_key_by_index(dict_index, dict_data):
    dict_index = int(dict_index)
    dict_key = ""
    if len(dict_data) < dict_index:
        return None

    dict_key = list(dict_data.keys())[dict_index - 1]
    return dict_key


def get_alias_from_line_name(line_name: str) -> str:
    line_alias = ""
    if line_name.find("_") > 0:
        line_alias = line_name.split("_")[1]
    return line_alias


def get_item_index_alias_from_catalog(item_index_alias, line_index_alias, catalog_data):
    line_index, line_alias = get_line_index_alias_from_catalog(
        line_index_alias=line_index_alias, catalog_data=catalog_data
    )
    line_name = get_dict_key_by_index(dict_index=line_index, dict_data=catalog_data)

    line_items = catalog_data[line_name]
    try:
        item_index = int(item_index_alias)
        item_index = at_least_1(item_index)
        if len(line_items) >= item_index:
            line_name = list(line_items.keys())[item_index - 1]
            item_alias = line_items[line_name]["name"]
        else:
            item_alias = None

    except:
        item_alias = str(item_index_alias)
        item_index = len(line_items) + 1

        for line_item_index, line_item_name in enumerate(line_items, start=1):
            if item_alias == line_items[line_item_name]["name"]:
                item_index = line_item_index
                break
        if item_index > len(line_items):
            item_alias = None

    return item_index, item_alias


def clean_catalogs():
    catalogs = os.listdir(folders.catalogs_folder())
    for catalog in catalogs:
        new_catalog_data = {}
        catalog_data = get_catalog_data(catalog)
        for line_name in catalog_data.keys():
            if len(catalog_data[line_name]) > 0:
                new_catalog_data[line_name] = catalog_data[line_name]

        if len(new_catalog_data) == 0:
            remove_catalog(catalog)
        else:
            set_catalog_data(data=new_catalog_data, catalog=catalog)


def remove_catalog(catalog):
    catalog_file = folders.catalogs_folder(path=catalog)
    catalog_file = files.apply_ext(file_path=catalog_file, ext="yaml")
    os.remove(catalog_file)



from bkmrks import urls
import pytest

def test_ensure_domain():
    tests = []
    tests.append({"url":"", "domain":"https://domain.com", "expected_result":"https://domain.com"})
    tests.append({"url":"/", "domain":"https://domain.com", "expected_result":"https://domain.com/"})
    tests.append({"url":"?param=1", "domain":"https://domain.com", "expected_result":"https://domain.com?param=1"})
    tests.append({"url":"/?param=1", "domain":"https://domain.com", "expected_result":"https://domain.com/?param=1"})
    tests.append({"url":"#anchor", "domain":"https://domain.com", "expected_result":"https://domain.com#anchor"})
    tests.append({"url":"/#anchor", "domain":"https://domain.com", "expected_result":"https://domain.com/#anchor"})
    tests.append({"url":"path", "domain":"https://domain.com", "expected_result":"https://domain.com/path"})
    tests.append({"url":"/path", "domain":"https://domain.com", "expected_result":"https://domain.com/path"})
    tests.append({"url":"/path", "domain":"https://domain.com", "expected_result":"https://domain.com/path"})
    tests.append({"url":"https://domain.com/path", "domain":"https://otherdomain.com", "expected_result":"https://domain.com/path"})
    tests.append({"url":"https://domain.com/path", "domain":"https://domain.com", "expected_result":"https://domain.com/path"})
    tests.append({"url":"https://domain.com/path", "domain":"", "expected_result":"https://domain.com/path"})
    tests.append({"url":"/path", "domain":"https://domain.com/other-path", "expected_result":"https://domain.com/path"})

    for test in tests:
        assert urls.ensure_domain(url=test["url"], domain=test["domain"]) == test["expected_result"]

    with pytest.raises(TypeError):
        urls.ensure_domain(url="/path")

    with pytest.raises(ValueError):
        urls.ensure_domain(url="/path",domain="not-a-domain.com")

def test_get_name_from_domain():
    with pytest.raises(TypeError):
        urls.get_name_from_domain()
    with pytest.raises(ValueError):
        urls.get_name_from_domain(url="testing.com.br")

    tests = []
    tests.append({"expected_result":"testing",         "url":"http://www.testing.com.br"})
    tests.append({"expected_result":"testing",         "url":"http://testing.com.br"})
    tests.append({"expected_result":"testing",         "url":"http://testing.com"})
    tests.append({"expected_result":"testing",         "url":"http://testing.cc"})
    tests.append({"expected_result":"google_gservice", "url":"http://gservice.google.com"})
    tests.append({"expected_result":"google_gservice", "url":"http://subdomain.gservice.google.com"})
    for test in tests:
        assert urls.get_name_from_domain(url=test["url"]) == test["expected_result"]

def test_extract_domain_from_url():
    with pytest.raises(TypeError):
        urls.extract_domain_from_url()

    tests = []
    tests.append({"url":"http://www.testing.com.br", "expected_result":"http://www.testing.com.br"})
    tests.append({"url":"http://www.testing.com.br/path?param1#anchor", "expected_result":"http://www.testing.com.br"})
    tests.append({"url":"www.testing.com.br", "expected_result":""})
    tests.append({"url":"/asdf", "expected_result":""})

    for test in tests:
        assert urls.extract_domain_from_url(url=test["url"]) == test["expected_result"]

def test_read_from_url_or_path():
    #TODO: To implement
    return



import urllib

import requests


def ensure_domain(url, domain):
    if (
        urllib.parse.urlparse(url).netloc == ""
        and urllib.parse.urlparse(domain).netloc == ""
    ):
        raise ValueError("`url` or `domain` must be a domain")

    if urllib.parse.urlparse(url).netloc == "":
        parsed_url = urllib.parse.urlparse(url)
        parsed_domain = urllib.parse.urlparse(domain)
        url = parsed_url._replace(scheme="https", netloc=parsed_domain.netloc).geturl()

    url = urllib.parse.urlparse(url)._replace(scheme="https").geturl()
    return url


def get_name_from_domain(url):
    href_parse = urllib.parse.urlparse(url)
    country_code_len = 2

    if href_parse.netloc == "":
        raise ValueError("`url` must have a domain")
    name = href_parse.netloc.split(".")

    if len(name[-1]) == country_code_len:
        name = name[:-1]

    if name[0] == "www":
        name = name[1:]

    if len(name) > 2:
        name = "_".join(name[-3:-1][::-1])
    elif len(name) == 1:
        name = name[0]
    else:
        name = name[-2]
    return name


def extract_domain_from_url(url):
    domain = ""

    url_parse = urllib.parse.urlparse(url)
    if len(url_parse.netloc) > 0:
        domain = "://".join([url_parse.scheme, url_parse.netloc])
    return domain


def read_from_url_or_path(url_path):
    if str(url_path).startswith("https://"):
        try:
            content = requests.get(url_path).text
        except:
            return ""
    else:
        url_path = url_path.split(".")[0] + ".html"
        with open(url_path, "r") as f:
            content = f.read()
    return content


def ensure_relative_path(path, url):
    if path[0] == "/":
        return ensure_domain(url=path, domain=url)

    if len(extract_domain_from_url(path)) > 0:
        return path

    parsed_url = urllib.parse.urlparse(url)
    last_url_element = parsed_url.path.split("/")[-1]
    if last_url_element.find(".") >= 0:
        parsed_url_path = parsed_url.path.split("/")[:-1]
    else:
        parsed_url_path = parsed_url.path.split("/")
    url = parsed_url._replace(
        path="/".join(parsed_url_path), query="", params="", fragment=""
    ).geturl()

    url = f"{url}/{path}"
    return url
